/*
** Module   : ConfigHelper.java
** Abstract : Implements logic around config fields. 
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20150323 Created initial version
** 002 VIG 20150430 Changed setModified signature: WidgetConfig instead of BaseConfig.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.ui.client.*;

/**
 * The class contains methods implementing elementary logic around the
 * {@link BaseConfig} fields.
 * 
 */
public class ConfigHelper
{
   /**
    * Returns the effective COLUMN value from the passed config instance.
    * The returned value represents the actual widget location as rendered
    * on the client.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    *          
    * @return  COLUMN character value. 
    */
   public static double getEffectiveColumn(BaseConfig cfg)
   {
      double col = cfg.clientColumn;
      
      if (col == BaseConfig.INV_COORD)
      {
         col = getAssignedColumn(cfg);
      }
      
      if (col == BaseConfig.INV_COORD)
      {
         col = 0;
      }
      
      return col;
   }
   
   /**
    * Returns the effective ROW value from the passed config instance.
    * The returned value represents the actual widget location as rendered
    * on the client.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    *          
    * @return  ROW character value. 
    */
   public static double getEffectiveRow(BaseConfig cfg)
   {
      double row = cfg.clientRow;
      
      if (row == BaseConfig.INV_COORD)
      {
         row = getAssignedRow(cfg);
      }
      
      if (row == BaseConfig.INV_COORD)
      {
         row = 0;
      }
      
      return row;
   }
   
   /**
    * The returned value represents the actual widget location as rendered
    * on the client.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    *          
    * @return  A valid {@link Point} reference. 
    */
   public static Point getEffectiveLocation(BaseConfig cfg)
   {
      return new Point(getEffectiveColumn(cfg), getEffectiveRow(cfg));
   }
   
   /**
    * Returns the effective X value from the passed config instance.
    * The returned value represents the actual widget location as rendered
    * on the client.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    *          
    * @return  X coordinate value. 
    */
   public static int getEffectiveX(BaseConfig cfg)
   {
      int x = cfg.clientX;
      
      if (x == BaseConfig.INV_COORD)
      {
         x = getAssignedX(cfg);
      }
      
      if (x == BaseConfig.INV_COORD)
      {
         x = 0;
      }
      
      return x;
   }
   
   /**
    * Returns the effective Y value from the passed config instance.
    * The returned value represents the actual widget location as rendered
    * on the client.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    *          
    * @return  Y coordinate value. 
    */
   public static int getEffectiveY(BaseConfig cfg)
   {
      int y = cfg.clientY;
      
      if (y == BaseConfig.INV_COORD)
      {
         y = getAssignedY(cfg);
      }
      
      if (y == BaseConfig.INV_COORD)
      {
         y = 0;
      }
      
      return y;
   }
   
   /**
    * The returned value represents the actual widget location as rendered
    * on the client.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    *          
    * @return  A valid {@link NativePoint} reference. 
    */
   public static NativePoint getEffectivePixelLocation(BaseConfig cfg)
   {
      return new NativePoint(getEffectiveX(cfg), getEffectiveY(cfg));
   }
   
   /**
    * Returns the initial COLUMN value from the passed config
    * instance. The returned value represents the value assigned in
    * frame definition.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  COLUMN character value.
    */
   public static double getInitialColumn(BaseConfig cfg)
   {
      if (!isLeftAligned(cfg))
      {
         return BaseConfig.INV_COORD;
      }
      
      return cfg.initColumn;
   }
   
   /**
    * Returns the initial ROW value from the passed config
    * instance. The returned value represents the value assigned in
    * frame definition.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  ROW character value.
    */
   public static double getInitialRow(BaseConfig cfg)
   {
      return cfg.initRow;
   }
   
   /**
    * The returned value represents the location value assigned in
    * frame definition.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  A valid {@link Point} reference.
    */
   public static Point getInitialLocation(BaseConfig cfg)
   {
      return new Point(getInitialColumn(cfg), getInitialRow(cfg));
   }
   
   /**
    * Returns the initial X value from the passed config instance. The
    * returned value represents the value assigned in frame
    * definition.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  X coordinate value.
    */
   public static int getInitialX(BaseConfig cfg)
   {
      if (!isLeftAligned(cfg))
      {
         return BaseConfig.INV_COORD;
      }
      
      return cfg.initX;
   }
   
   /**
    * Returns the initial Y value from the passed config instance. The
    * returned value represents the value assigned in frame
    * definition.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  Y coordinate value.
    */
   public static int getInitialY(BaseConfig cfg)
   {
      return cfg.initY;
   }
   
   /**
    * The returned value represents the location value assigned in
    * frame definition.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  A valid {@link NativePoint} reference.
    */
   public static NativePoint getInitialPixelLocation(BaseConfig cfg)
   {
      return new NativePoint(getInitialX(cfg), getInitialY(cfg));
   }
   
   /**
    * Returns the initial or set COLUMN value from the passed config
    * instance. The returned value represents the value assigned in
    * frame definition or as set by COLUMN attribute.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  COLUMN character value.
    */
   public static double getAssignedColumn(BaseConfig cfg)
   {
      if (!isLeftAligned(cfg))
      {
         return BaseConfig.INV_COORD;
      }
      
      return cfg.column != BaseConfig.INV_COORD ? cfg.column : cfg.initColumn;
   }
   
   /**
    * Returns the initial or set ROW value from the passed config
    * instance. The returned value represents the value assigned in
    * frame definition or as set by ROW attribute.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  ROW character value.
    */
   public static double getAssignedRow(BaseConfig cfg)
   {
      return cfg.row != BaseConfig.INV_COORD ? cfg.row : cfg.initRow;
   }
   
   /**
    * The returned value represents the location value assigned in
    * frame definition or as set by COLUMN/ROW attributes.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  A valid {@link Point} reference.
    */
   public static Point getAssignedLocation(BaseConfig cfg)
   {
      return new Point(getAssignedColumn(cfg), getAssignedRow(cfg));
   }
   
   /**
    * Returns the initial or set X value from the passed config
    * instance. The returned value represents the value assigned in
    * frame definition or as set by X attribute.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  X pixel value.
    */
   public static int getAssignedX(BaseConfig cfg)
   {
      if (!isLeftAligned(cfg))
      {
         return BaseConfig.INV_COORD;
      }
      
      return cfg.x != BaseConfig.INV_COORD ? cfg.x : cfg.initX;
   }
   
   /**
    * Returns the initial or set Y value from the passed config
    * instance. The returned value represents the value assigned in
    * frame definition or as set by Y attribute.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  Y pixel value.
    */
   public static int getAssignedY(BaseConfig cfg)
   {
      return cfg.y != BaseConfig.INV_COORD ? cfg.y : cfg.initY;
   }
   
   /**
    * The returned value represents the location value assigned in
    * frame definition or as set by X/Y attributes.
    * 
    * @param   cfg
    *          A valid {@link BaseConfig} reference.
    * 
    * @return  A valid {@link NativePoint} reference.
    */
   public static NativePoint getAssignedPixelLocation(BaseConfig cfg)
   {
      return new NativePoint(getAssignedX(cfg), getAssignedY(cfg));
   }
   
   /**
    * Check if widget is left aligned.
    * 
    * @return  <code>true</code> if widget is left aligned.
    */
   public static boolean isLeftAligned(BaseConfig cfg)
   {
      return cfg.align == ControlEntity.ALIGN_LEFT;
   }
   
   /**
    * Get COLON attribute value from the passed configuration. Note 
    * that Progress does not have corresponding attribute accessible 
    * to program.
    * 
    * @param    cfg
    *           A valid {@link BaseConfig} instance.
    * 
    * @return   Current attribute value.
    */
   public static double getTo(BaseConfig cfg)
   {
      if (cfg.align == ControlEntity.ALIGN_RIGHT)
      {
         return cfg.column != BaseConfig.INV_COORD ? cfg.column : cfg.initColumn;
      }
      
      return BaseConfig.INV_COORD;
   }

   /**
    * Sets the rightmost column of the widget in the passed configuration.
    * This method must be called only on the server.
    * 
    * @param    cfg
    *           A valid {@link BaseConfig} instance.
    * @param    value
    *           The 0-based column number.
    */
   public static void setTo(BaseConfig cfg, double value)
   {
      value = Coordinate.scale(value);
      
      if (value == BaseConfig.INV_COORD)
      {
         return;
      }
      
      cfg.initColumn = value;
      cfg.align = ControlEntity.ALIGN_RIGHT;
   }

   /**
    * Get COLON attribute value from the passed configuration. Note
    * that Progress does not have corresponding attribute accessible
    * to program.
    * 
    * @param cfg
    *        A valid {@link BaseConfig} instance.
    * 
    * @return Current attribute value.
    */
   public static double getColon(BaseConfig cfg)
   {
      if (cfg.align == ControlEntity.ALIGN_COLON)
      {
         return cfg.column != BaseConfig.INV_COORD ? cfg.column : cfg.initColumn;
      }
      
      return BaseConfig.INV_COORD;
   }

   /**
    * Set COLON attribute value in the passed configuration. Note that
    * Progress does not have corresponding attribute accessible to
    * program. This method must be called only on the server. 
    * 
    * @param cfg
    *        A valid {@link BaseConfig} instance.
    * 
    * @param colon
    *           New value for the attribute.
    */
   public static void setColon(BaseConfig cfg, double colon)
   {
      colon = Coordinate.scale(colon);
      
      if (colon == BaseConfig.INV_COORD)
         return;
      
      cfg.initColumn = colon;
      cfg.align = ControlEntity.ALIGN_COLON;
   }

   /**
    * Set new value to the MODIFIED attribute in the passed
    * configuration.
    * 
    * @param cfg
    *        A valid {@link BaseConfig} instance.
    * @param modified
    *        New value for the attribute.
    */
   public static void setModified(WidgetConfig cfg, boolean modified)
   {
      cfg.modified = modified;
      
      // TODO: should this always get unilaterally set or should this only get set when
      //       modified changes from false to true?
      cfg.state = ScreenBuffer.CHANGED;
   }
}
