/*
** Module   : SubMenuWidget.java
** Abstract : server side sub-menu widget implementation
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------------Description-----------------------------
** 001 VIG 20141124 Created initial version.
** 002 VIG 20150115 Changed base class, added initialization method.
** 003 VIG 20150302 Added name attribute setting, firstChild, lastChild methods.
**                  Added findSubMenuStatic implementation.
*/
package com.goldencode.p2j.ui;

import java.lang.reflect.*;

import com.goldencode.p2j.ui.client.FontManager;
import com.goldencode.p2j.util.*;

/**
 * This class represents sub-menu widget.
 */
@LegacyResource(resource = LegacyResource.SUB_MENU)
public class SubMenuWidget
extends MenuContainerWidget<SubMenuConfig>
{
   /**
    * Default constructor.  
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    */
   public SubMenuWidget(boolean dynamic)
   {
      super(dynamic, new SubMenuConfig());
   }

   /**
    * Gets this sub-menu parent (MENU or SUB-MENU).
    * 
    * @return  See above.
    */
   @Override
   public GenericWidget<?> getParent()
   {
      return LogicalTerminal.getWidgetForId(config.parentId);
   }

   /**
    * Sets this sub-menu parent (MENU or SUB-MENU).
    * 
    * @param  parent
    *         MENU or SUB-MENU widget.
    */
   @Override
   public void setParent(GenericWidget<?> parent)
   {
      if (parent instanceof MenuWidget || parent instanceof SubMenuWidget)
      {
         config.parentId = parent.getId();
      }
      else
      {
         // TODO "**Unable to assign UNKNOWN value to attribute PARENT on SUB-MENU widget. (4083)"
         String msgTemplate = "**%s widget cannot be a parent of a SUB-MENU widget";
         ErrorManager.recordOrThrowError(4075, String.format(msgTemplate, parent.type()), false);
      }
      
      pushScreenDefinition();
   }
   
   /**
    * Get the label text for this widget.
    *
    * @return   The label text.
    */
   public character getLabel()
   {
      return new character(config.title);
   }
   
   /**
    * Sets the label text, parsing mnemonic.
    * 
    * TODO: find out what is going on with undocumented feature of very long label
    * See parent's javadoc.
    * 
    * @param    label
    *           The new label text.
    */
   @Override
   public void setLabel(String label)
   {
      config.title = label;
      
      // TODO Check mnemonic support
   }
   
   /**
    * Gets the display color writable attribute.
    *
    * @return   The current value of the display color attribute.
    */
   @Override
   public integer getDColor()
   {
      int color = config.dcolor.color();
      return (color == -1) ? new integer() : new integer(color);
   }
   
   /**
    * Sets the display color.
    *
    * @param    param
    *           The display color.
    */
   public void setDColor(NumberType param)
   {
      config.dcolor = new ColorSpec(param.intValue()).convert();
      pushScreenDefinition();
   }

   /**
    * Sets the display color.
    * 
    * @param    displayColor
    *           The display color.
    */
   public void setDColor(Color displayColor)
   {
      config.dcolor = displayColor;
      pushScreenDefinition();
   }

   /**
    * Set widget DCOLOR
    * 
    * @param   color
    *          Integer color value.
    */
   public void setDColor(int color)
   {
      setDColor(new ColorSpec(color).convert());
   }
   
   /**
    * Gets the FGCOLOR writable attribute.
    *
    * @return   The current value of the FGCOLOR attribute.
    */
   public integer getFgColor()
   {
      int color = config.fgcolor;
      return color == -1 ? new integer() : new integer(color);
   }

   /**
    * Sets the FGCOLOR writable attribute.
    *
    * @param    fgcolor
    *           The new value for the FGCOLOR attribute.
    */
   @Override
   public void setFgColor(integer fgcolor)
   {
      int color = (fgcolor == null || fgcolor.isUnknown()) ? -1 : fgcolor.intValue();
      setFgColor(color);
   }   

   /**
    * Sets the FGCOLOR writable attribute.
    *
    * @param    fgcolor
    *           The new value for the FGCOLOR attribute.
    */
   public void setFgColor(int64 fgcolor)
   {
      int color = (fgcolor == null || fgcolor.isUnknown()) ? -1 : fgcolor.intValue(); 
      setFgColor(color);
   }

   /**
    * Sets the FGCOLOR writable attribute.
    *
    * @param    fgcolor
    *           The new value for the FGCOLOR attribute.
    */
   public void setFgColor(int fgcolor)
   {
      config.fgcolor = fgcolor;
      pushScreenDefinition();            
   }

   /**
    * Gets the BGCOLOR writable attribute.
    *
    * @return   The current value of the BGCOLOR attribute.
    */
   public integer getBgColor()
   {
      int color = config.bgcolor;
      return (color == -1) ? new integer() : new integer(color);
   }
   
   /**
    * Sets the BGCOLOR writable attribute.
    *
    * @param    bgcolor
    *           The new value for the BGCOLOR attribute.
    */
   @Override
   public void setBgColor(integer bgcolor)
   {
      int color = (bgcolor == null || bgcolor.isUnknown()) ? -1 : bgcolor.intValue();
      setBgColor(color);
   }

   /**
    * Sets the BGCOLOR writable attribute.
    *
    * @param    bgcolor
    *           The new value for the BGCOLOR attribute.
    */
   public void setBgColor(int64 bgcolor)
   {
      int color = (bgcolor == null || bgcolor.isUnknown()) ? -1 : bgcolor.intValue(); 
      setBgColor(color);
   }

   /**
    * Sets the BGCOLOR writable attribute.
    *
    * @param    bgcolor
    *           The new value for the BGCOLOR attribute.
    */
   public void setBgColor(int bgcolor)
   {
      config.bgcolor = bgcolor;
      pushScreenDefinition();            
   }
   
   /**
    * Gets the input (prompt/edit mode) color writable attribute.
    *
    * @return   The current value of the input (prompt/edit mode) color attribute.
    */
   @Override
   public integer getPfColor()
   {
      int color = config.pfcolor.color();
      return (color == -1) ? new integer() : new integer(color);
   }
   
   /**
    * Sets the input (prompt/edit mode) color.
    *
    * @param    param
    *           The color to use.
    */
   public void setPfColor(NumberType param)
   {
      config.pfcolor = new ColorSpec(param.intValue()).convert();
      pushScreenDefinition();
   }

   /**
    * Sets the input (prompt/edit mode) color.
    * 
    * @param    inputColor
    *           The color to use for when in "input" mode.
    */
   public void setPfColor(Color inputColor)
   {
      config.pfcolor = inputColor;
      pushScreenDefinition();
   }

   /**
    * Set widget PFCOLOR
    * 
    * @param   color
    *          Integer color value.
    */
   public void setPfColor(int color)
   {
      setPfColor(new ColorSpec(color).convert());
   }
   
   /**
    * Set the FONT attribute of this widget.
    * 
    * @param    fontNum
    *           An entry in the font-table or unknown to refer the default font.
    */
   @Override
   public void setFont(int64 fontNum)
   {
      // if the font is unknown, it gets set to UNDEFINED.
      int font = (fontNum.isUnknown() ? FontManager.UNDEFINED_FONT : fontNum.intValue());

      config.font = font;
      pushScreenDefinition();
   }
   
   /**
    * Get the FONT attribute of this widget.
    * 
    * @return   A value representing an entry in the user's font-table.
    */
   @Override
   public integer getFont()
   {
      int font = config.font;
      
      return (font == FontManager.UNDEFINED_FONT ? new integer() : new integer(font));
   }

   /**
    * Finds menu-item in this menu tree. It can appear on different levels.
    * Menu-item is searched using some sort of depth-first algorithm.
    * See {@link MenuWidget#findMenuItemInt(String)}
    * 
    * @param   name
    *          MENU-ITEM name.
    *          
    * @return  concrete {@link MenuItemWidget} or null if not found.
    */
   public MenuItemWidget findMenuItem(String name)
   {
      return findMenuItemInt(name);
   }
   
   /**
    * Finds sub-menu in this sub-menu tree. It can appear only once. 
    * Simple depth-first search is used.
    * 
    * @param   name
    *          Sub-menu name.
    * 
    * @return  concrete {@link SubMenuWidget} or null if not found.
    */
   public SubMenuWidget findSubMenu(String name)
   {
      return findSubMenuInt(name);
   }
   
   /**
    * Accesses the first window in the list of contained windows, having this window as parent.
    *
    * @return   The first widget or <code>null</code> if no widgets are contained.
    */
   @Override
   public handle firstChild()
   {
      return HandleChain.firstResource(this, LegacyResource.SUB_MENU);
   }
   
   /**
    * Accesses the last window in the list of contained windows, having this window as parent.
    *
    * @return   The last widget or <code>null</code> if no widgets are contained.
    */
   @Override
   public handle lastChild()
   {
      return HandleChain.lastResource(this, LegacyResource.SUB_MENU);
   }
   
   /**
    * Creates new sub-menu as a proxy object (as a local resource).  This loads
    * the static sub-menu definition.
    *
    * @param    <T> 
    *           Sub-menu definition class type.
    * @param    defCls
    *           The class of sub-menu definition.
    * @param    name
    *           The name of sub-menu.          
    *
    * @return   The created sub-menu widget.
    */
   public static <T extends SubMenuDefinition> T createStaticSubMenu(Class<T> defCls, String name)
   {
      SubMenuWidget subMenu = new SubMenuWidget(false);
      subMenu.name(name);
      
      final T menuDef;
      try
      {
         menuDef = defCls.getConstructor(new Class<?>[]{}).newInstance();
         subMenu.config.id = new WidgetId(LogicalTerminal.registerMenu(subMenu));
         subMenu.config.name = name;
         menuDef.setSelf(subMenu);
         menuDef.setup(subMenu);
         subMenu.coreInitialize(defCls, menuDef, false);
         subMenu.batch(false, true);
         //subMenu.finishConfigProcessing();
      }
      catch (NoSuchMethodException    | 
             SecurityException        | 
             InstantiationException   | 
             IllegalAccessException   | 
             IllegalArgumentException | 
             InvocationTargetException e)
      {
         throw new RuntimeException("Failed to instantiate sub-menu definition class", e);
      }
      
      return menuDef;
   }
   
   /**
    * Finds the first SUB-MENU with specified name in all menus.
    * 
    * @param   name
    *          The name of searched SUB-MENU
    *          
    * @return  The first found SubMenuWidget or null.
    */
   public static SubMenuWidget findSubMenuStatic(String name)
   {
      for (MenuWidget menu : LogicalTerminal.getAllMenus())
      {
         SubMenuWidget found = menu.findSubMenu(name);
         if (found != null)
         {
            return found;
         }
      }
      
      return null;
   }
   
}
